#include "config.h"
//#include "nfs/nfs.h"
#include "nfslib.h"
#include "nfspdctl.h"

#define NFSPCTL_FLUSH		0
#define NFSPCTL_DROP		1
#define NFSPCTL_FORW		2
#define NFSPCTL_ADDIOD		3
#define NFSPCTL_FREEIOD		4
#define NFSPCTL_BIND		5
#define NFSPCTL_UNBIND		6
#define NFSPCTL_SETNBV		7
#define NFSPCTL_GETNBV		8
#define NFSPCTL_GETVIODSZ	9

/* for compatibility with nfsp user-mode */
int nfspd_initialize() {
	return 0;
}

/* idem */
int nfspd_terminate() {
	return 0;
}

/* creates a new iod descriptor given its address and port */
int nfspd_addiod(const struct sockaddr_in *addr) {
	struct nfsctl_arg args;
	union nfsctl_res res;
	int err;
	
	args.ca_version = NFSCTL_VERSION;
	args.ca_nfsp.cap_fnct = NFSPCTL_ADDIOD;
	args.ca_nfsp.cap_addr = (struct sockaddr_in *)addr;
	err = nfsctl(NFSCTL_NFSP, &args, &res);
	if (err) {
		return -1;
	}
	return res.cr_nfsp;
}

/* free an iod descriptor */
int nfspd_freeiod(int ih) {
	struct nfsctl_arg args;

	args.ca_version = NFSCTL_VERSION;
	args.ca_nfsp.cap_fnct = NFSPCTL_FREEIOD;
	args.ca_nfsp.cap_ih = ih;
	return nfsctl(NFSCTL_NFSP, &args, NULL);
}

/* bind an iod to a viod */
int nfspd_bindiod(int iviod, int ih) {
	struct nfsctl_arg args;

	args.ca_version = NFSCTL_VERSION;
	args.ca_nfsp.cap_fnct = NFSPCTL_BIND;
	args.ca_nfsp.cap_iviod = iviod;
	args.ca_nfsp.cap_ih = ih;
	return nfsctl(NFSCTL_NFSP, &args, NULL);
}

/* unbind an iod to a viod */
int nfspd_unbindiod(int ih) {
	struct nfsctl_arg args;

	args.ca_version = NFSCTL_VERSION;
	args.ca_nfsp.cap_fnct = NFSPCTL_UNBIND;
	args.ca_nfsp.cap_ih = ih;
	return nfsctl(NFSCTL_NFSP, &args, NULL);
}

/* sets nbviod */
int nfspd_setnbviods(int nb) {
	struct nfsctl_arg args;

	args.ca_version = NFSCTL_VERSION;
	args.ca_nfsp.cap_fnct = NFSPCTL_SETNBV;
	args.ca_nfsp.cap_nbviods = nb;
	return nfsctl(NFSCTL_NFSP, &args, NULL);
}

/* gets the viods number */
int nfspd_getnbviods() {
	struct nfsctl_arg args;
	union nfsctl_res res;
	int err;

	args.ca_version = NFSCTL_VERSION;
	args.ca_nfsp.cap_fnct = NFSPCTL_GETNBV;
	err = nfsctl(NFSCTL_NFSP, &args, &res);
	if (err) {
		return -1;
	}
	return res.cr_nfsp;
}

/* asks the nfspd to go in DROP mode */
int nfspd_drop() {
	struct nfsctl_arg args;

	args.ca_version = NFSCTL_VERSION;
	args.ca_nfsp.cap_fnct = NFSPCTL_DROP;
	return nfsctl(NFSCTL_NFSP, &args, NULL);
}

/* asks the nfspd to go in FORWARD mode */
int nfspd_forward() {
	struct nfsctl_arg args;

	args.ca_version = NFSCTL_VERSION;
	args.ca_nfsp.cap_fnct = NFSPCTL_FORW;
	return nfsctl(NFSCTL_NFSP, &args, NULL);
}

/* asks the nfspd to reset its mode (a.k.a. FLUSH operation) */
int nfspd_flush() {
	struct nfsctl_arg args;

	args.ca_version = NFSCTL_VERSION;
	args.ca_nfsp.cap_fnct = NFSPCTL_FLUSH;
	return nfsctl(NFSCTL_NFSP, &args, NULL);
}

/* ask the nfspd how many iods are in the viod */
int nfspd_get_viod_size(const int iviod) {
	struct nfsctl_arg args;
	union nfsctl_res res;
	int err;

	args.ca_version = NFSCTL_VERSION;
	args.ca_nfsp.cap_fnct = NFSPCTL_GETVIODSZ;
	args.ca_nfsp.cap_iviod = iviod;
	err = nfsctl(NFSCTL_NFSP, &args, &res);
	if (err) {
		return -1;
	}
	return res.cr_nfsp;
}
