// udpspoofng header
// defines the API used for spoofing UDP packets
// 
// Changelog:
// 200205xx Adrien Lebre <adrien.lebre@imag.fr>
//   - IP_send() changed into ETHERNET_send() as we need to fill the MAC
//   destination field
//
// Author:
// 2001xxxx Pierre Lombard <pierre.lombard@imag.fr>
//
// Licence: GPL 

#ifndef __UDPSPOOF_H__
#define __UDPSPOOF_H__

#include <netinet/ip.h>

// Inits the udpspoof library : sets up routing information (lo/eth0)
// MUST be called prior to any use
void udpspoof_init();

extern unsigned int eth0addr;

// Inits the IP sequence number
// 
// A default value is set BUT every process (in the case of a fork) should
// call this function to have different values of IP sequences
void set_ipid(const unsigned short int x);

//typedef void (*packet_send_func_t)(struct iphdr * const, const int, const char * const); 

// Sends the data contained in the pseudo IP packet
//  pseudoip: refers an struct iphdr followed by an struct udphdr and data
//			  UDP packet MUST be complete (especially size)
//			  IP pseudo header MUST be of size sizeof(struct iphdr) and 
//			  have src & dest address correctly set
//  sockraw : the IPPROTO_RAW socket used to send packets
void IP_send(struct iphdr * const pseudoip, const int sockraw);


// Parameters:
// 
//  pseudoip: refers a struct iphdr followed by an struct udphdr UDP and the data
//            - UDP packet MUST be complete (esp. size and checksum)
//            - IP pseudo header MUST be of size sizeof(struct iphdr) and have
//            its source and address field set
//            
//  dest_mac_addr: MAC address of the destination host (matching with the IP
//  address set in destination field)
//
//  s: an open SOCK_DGRAM socket with ETH_P_ALL (link layer level socket - man 7 packet)
//
// WARNING:
//  This send is destructive on the buffer pointed to by pseudoip.
//  IOW, DO NOT USE THE BUFFER AFTER THIS FUNCTION.
void ETHERNET_send(struct iphdr * const pseudoip, const int s, const char * const dest_mac_addr);


void UDP_send(struct iphdr * const pseudoip, const int s);
#endif
